@@ -4,25 +4,19 @@ Emulate Javascript object in Python those object will be converted to their Java
 
 At least the following doesn't work
 
-- print(foo.bar) doesn't work
-- call without statement doesn't work do always assign things
 - functions don't take keyword arguments
 - args, **kwargs is not supported
 - assignements support only one target
 """
 import sys
+from types import GeneratorType
+
 from ast import parse
 from ast import NodeVisitor
 
 
 class JSObject(dict):
 
-    def __getattr__(self, key):
-        return self[key]
-
-    def __setattr__(self, key, value):
-        self[key] = value
-
     def length(self):
         return len(self)
 
@@ -38,22 +32,22 @@ class JSArray(list):
 
 class JSGenerator(NodeVisitor):
 
-    def generic_visit(self, node):
-        return super(JSGenerator, self).generic_visit(node)
-
-    def visit(self, node):
-        return super(JSGenerator, self).visit(node)
-
     def visit_Module(self, node):
         return '\n'.join(map(self.visit, node.body))
 
     def visit_FunctionDef(self, node):
-        args = self.visit(node.args)  # FIXME: support kwargs
+        args = self.visit(node.args)
         buffer = 'var %s = function(%s) {\n' % (
             node.name,
             ', '.join(args),
         )
-        body = map(self.visit, node.body)
+        body = list()
+        for child in node.body:
+            if isinstance(child, GeneratorType):
+                for sub in child:
+                    body.append(self.visit(sub))
+            else:
+                body.append(self.visit(child))
         buffer += '\n'.join(body)
         buffer += '\n}\n'
         return buffer
@@ -61,16 +55,12 @@ class JSGenerator(NodeVisitor):
     def visit_arguments(self, node):
         out = []
         for name in [self.visit(arg) for arg in node.args]:
-            out.append(name)  # FIXME: support kwargs
+            out.append(name)
         return out
 
     def visit_Name(self, node):
         if node.id == 'None':
             return 'undefined'
-        if node.id == 'JSObject':
-            return 'Object'
-        if node.id == 'JSArray':
-            return 'new Array'
         return node.id
 
     def visit_Attribute(self, node):
@@ -84,40 +74,29 @@ class JSGenerator(NodeVisitor):
         return s
 
     def visit_keyword(self, node):
-        return node.arg, self.visit(node.value)
+        if isinstance(node.arg, basestring):
+            return node.arg, self.visit(node.value)
+        return self.visit(node.arg), self.visit(node.value)
 
     def visit_Call(self, node):
         name = self.visit(node.func)
-        if name.endswith('___get'):
-            object = node.func.value.id
-            arg = node.args[0].id
-            return '%s[%s]' % (object, arg)
-        elif name.endswith('___set'):
-            object = node.func.value.id
-            key = node.args[0].id
-            value = node.args[1].id
-            return '%s[%s] = %s' % (object, key, value)
-        elif name.endswith('___insert'):
-            object = node.func.value.id
-            pos = node.args[0].n
-            item = node.args[1].id
-            return 'Array.prototype.splice.apply(%s, [%s, 0, %s])' % (object, pos, item)
-        elif name.endswith('___slice'):
-            object = node.func.value.id
-            start = node.args[0].n
-            if len(node.args) > 1:
-                end = node.args[1].n
-                return 'Array.prototype.slice.call(%s).slice(%s, %s)' % (object, start, end)
-            return 'Array.prototype.slice.call(%s).slice(%s)' % (object, start)
-        elif name == 'dict':
-            kwargs = map(self.visit, node.keywords)
-            f = lambda x: '"%s": %s' % (x[0], x[1])
-            out = ', '.join(map(f, kwargs))
+        if name == 'JSObject':
+            if node.keywords:
+                kwargs = map(self.visit, node.keywords)
+                f = lambda x: '"%s": %s' % (x[0], x[1])
+                out = ', '.join(map(f, kwargs))
+            else:
+                out = ''
             return '{%s}' % out
+        if name == 'JSArray':
+            if node.args:
+                args = map(self.visit, node.args)
+                out = ', '.join(args)
+            else:
+                out = ''
+            return 'create_array(%s)' % out
         elif name == 'JS':
             return node.args[0].s
-        elif name == 'toString':
-            return 'Object().toString.call(%s)' % self.visit(node.args[0])
         else:
             if node.args:
                 args = [self.visit(e) for e in node.args]
@@ -148,7 +127,9 @@ class JSGenerator(NodeVisitor):
         return self.visit(node.value) + ';'
 
     def visit_Return(self, node):
-        return 'return %s;' % self.visit(node.value)
+        if node.value:
+            return 'return %s;' % self.visit(node.value)
+        return 'return undefined;'
 
     def visit_Pass(self, node):
         return ''
@@ -178,7 +159,7 @@ class JSGenerator(NodeVisitor):
 
     def visit_For(self, node):
         target = node.target.id
-        num = self.visit(node.iter.args[0])
+        num = self.visit(node.iter)
         body = '\n'.join(map(self.visit, node.body)) + '\n'
         return 'for (%s=0; %s<%s; %s++) {\n%s}\n' % (target, target, num, target, body)
 
